@@ -964,8 +964,11 @@ class Connection implements ServerVersionProvider
             $res = $func($this);
 
             $successful = true;
+        } catch (ConnectionLost $connectionLost) {
+            // Catching here only to be able to prevent a rollback attempt
+            throw $connectionLost;
         } finally {
-            if (! $successful) {
+            if (! isset($connectionLost) && ! $successful) {
                 $this->rollBack();
             }
         }
@@ -981,6 +984,7 @@ class Connection implements ServerVersionProvider
                 || $t instanceof UniqueConstraintViolationException
                 || $t instanceof ForeignKeyConstraintViolationException
                 || $t instanceof DeadlockException
+                || $t instanceof ConnectionLost
             );
 
             throw $t;
