@@ -25,8 +25,9 @@ use PHPUnit\Framework\MockObject\MockObject;
 use PHPUnit\Framework\TestCase;
 use Psr\Cache\CacheItemInterface;
 use Psr\Cache\CacheItemPoolInterface;
+use RuntimeException;
 
-/** @psalm-import-type Params from DriverManager */
+/** @phpstan-import-type Params from DriverManager */
 #[RequiresPhpExtension('pdo_mysql')]
 class ConnectionTest extends TestCase
 {
@@ -47,11 +48,11 @@ class ConnectionTest extends TestCase
 
     private function getExecuteStatementMockConnection(): Connection&MockObject
     {
-        $driverMock = $this->createMock(Driver::class);
+        $driver = self::createStub(Driver::class);
 
         return $this->getMockBuilder(Connection::class)
             ->onlyMethods(['executeStatement'])
-            ->setConstructorArgs([[], $driverMock])
+            ->setConstructorArgs([[], $driver])
             ->getMock();
     }
 
@@ -146,9 +147,9 @@ class ConnectionTest extends TestCase
 
     public function testConnectStartsTransactionInNoAutoCommitMode(): void
     {
-        $driverMock = $this->createMock(Driver::class);
+        $driver = self::createStub(Driver::class);
 
-        $conn = new Connection([], $driverMock);
+        $conn = new Connection([], $driver);
 
         $conn->setAutoCommit(false);
 
@@ -159,12 +160,63 @@ class ConnectionTest extends TestCase
         self::assertTrue($conn->isTransactionActive());
     }
 
-    public function testCommitStartsTransactionInNoAutoCommitMode(): void
+    public function testTransactionIsNotActiveAfterTransactionalInAutoCommitMode(): void
     {
-        $driverMock = $this->createMock(Driver::class);
+        $driverMock = self::createStub(Driver::class);
+        $driverMock
+            ->method('connect')
+            ->willReturn(
+                self::createStub(DriverConnection::class),
+            );
+
+        $conn = new Connection([], $driverMock);
+
+        $conn->setAutoCommit(true);
+
+        self::assertFalse($conn->isTransactionActive());
+
+        $conn->transactional(static function (Connection $connection): void {
+        });
+
+        self::assertFalse($conn->isTransactionActive());
+    }
+
+    public function testTransactionIsActiveAfterTransactionalInNoAutoCommitMode(): void
+    {
+        $platformMock = self::createStub(AbstractPlatform::class);
+        $platformMock
+            ->method('supportsSavepoints')
+            ->willReturn(true);
+
+        $driverMock = self::createStub(Driver::class);
+        $driverMock
+            ->method('connect')
+            ->willReturn(
+                self::createStub(DriverConnection::class),
+            );
+
+        $driverMock
+            ->method('getDatabasePlatform')
+            ->willReturn(
+                $platformMock,
+            );
 
         $conn = new Connection([], $driverMock);
 
+        $conn->setAutoCommit(false);
+
+        $conn->transactional(static function (Connection $connection): void {
+        });
+
+        self::assertTrue($conn->isTransactionActive());
+    }
+
+    public function testCommitStartsTransactionInNoAutoCommitMode(): void
+    {
+        $driver = self::createStub(Driver::class);
+
+        $conn = new Connection([], $driver);
+
         $conn->setAutoCommit(false);
         $conn->executeQuery('SELECT 1');
         $conn->commit();
@@ -172,6 +224,36 @@ class ConnectionTest extends TestCase
         self::assertTrue($conn->isTransactionActive());
     }
 
+    public function testBeginTransactionFailureAfterCommitInNoAutoCommitMode(): void
+    {
+        $driverConnectionMock = $this->createMock(DriverConnection::class);
+        $driverConnectionMock->expects(self::exactly(2))
+            ->method('beginTransaction')
+            ->willReturnOnConsecutiveCalls(
+                true,
+                self::throwException(new RuntimeException()),
+            );
+
+        $driver = self::createStub(Driver::class);
+        $driver
+            ->method('connect')
+            ->willReturn(
+                $driverConnectionMock,
+            );
+        $conn = new Connection([], $driver);
+
+        $conn->setAutoCommit(false);
+
+        /** Invoke protected {@see Connection::connect()} */
+        $conn->getServerVersion();
+        try {
+            $conn->commit();
+        } catch (RuntimeException) {
+        }
+
+        self::assertTrue($conn->isTransactionActive());
+    }
+
     /** @return bool[][] */
     public static function resultProvider(): array
     {
@@ -180,9 +262,9 @@ class ConnectionTest extends TestCase
 
     public function testRollBackStartsTransactionInNoAutoCommitMode(): void
     {
-        $driverMock = $this->createMock(Driver::class);
+        $driver = self::createStub(Driver::class);
 
-        $conn = new Connection([], $driverMock);
+        $conn = new Connection([], $driver);
 
         $conn->setAutoCommit(false);
         $conn->executeQuery('SELECT 1');
@@ -198,12 +280,12 @@ class ConnectionTest extends TestCase
             ->method('supportsSavepoints')
             ->willReturn(true);
 
-        $driverMock = $this->createMock(Driver::class);
-        $driverMock
+        $driver = self::createStub(Driver::class);
+        $driver
             ->method('getDatabasePlatform')
             ->willReturn($platform);
 
-        $conn = new Connection([], $driverMock);
+        $conn = new Connection([], $driver);
 
         $conn->beginTransaction();
         $conn->beginTransaction();
@@ -374,6 +456,7 @@ class ConnectionTest extends TestCase
         );
     }
 
+    /** @param non-empty-string $method */
     #[DataProvider('fetchModeProvider')]
     public function testFetch(string $method, callable $invoke, mixed $expected): void
     {
@@ -387,7 +470,7 @@ class ConnectionTest extends TestCase
             ->willReturn($expected);
 
         $driver = $this->createConfiguredMock(Driver::class, [
-            'connect' => $this->createMock(DriverConnection::class),
+            'connect' => self::createStub(DriverConnection::class),
         ]);
 
         $conn = $this->getMockBuilder(Connection::class)
@@ -482,7 +565,7 @@ class ConnectionTest extends TestCase
             ->method('getServerVersion')
             ->willReturn('6.6.6');
 
-        $platform = $this->createMock(AbstractPlatform::class);
+        $platform = self::createStub(AbstractPlatform::class);
 
         $driver = $this->createMock(Driver::class);
         $driver->expects(self::once())
@@ -507,7 +590,7 @@ class ConnectionTest extends TestCase
         $driverConnection->expects(self::never())
             ->method('getServerVersion');
 
-        $platform = $this->createMock(AbstractPlatform::class);
+        $platform = self::createStub(AbstractPlatform::class);
 
         $driver = $this->createMock(Driver::class);
         $driver->expects(self::never())
@@ -525,9 +608,9 @@ class ConnectionTest extends TestCase
     {
         $driverMock = $this->createMock(Driver::class);
 
-        $driverConnectionMock = $this->createMock(Driver\Connection::class);
+        $driverConnectionMock = self::createStub(Driver\Connection::class);
 
-        $platformMock = $this->createMock(AbstractPlatform::class);
+        $platformMock = self::createStub(AbstractPlatform::class);
 
         $connection = new Connection(['serverVersion' => '8.0'], $driverMock);
 
@@ -547,9 +630,9 @@ class ConnectionTest extends TestCase
     {
         $driverMock = $this->createMock(Driver::class);
 
-        $driverConnectionMock = $this->createMock(Driver\Connection::class);
+        $driverConnectionMock = self::createStub(Driver\Connection::class);
 
-        $platformMock = $this->createMock(AbstractPlatform::class);
+        $platformMock = self::createStub(AbstractPlatform::class);
 
         $connection = new Connection(['primary' => ['serverVersion' => '8.0']], $driverMock);
 
@@ -567,9 +650,9 @@ class ConnectionTest extends TestCase
 
     public function testConnectionParamsArePassedToTheQueryCacheProfileInExecuteCacheQuery(): void
     {
-        $cacheItemMock = $this->createMock(CacheItemInterface::class);
+        $cacheItemMock = self::createStub(CacheItemInterface::class);
         $cacheItemMock->method('isHit')->willReturn(true);
-        $cacheItemMock->method('get')->willReturn(['realKey' => []]);
+        $cacheItemMock->method('get')->willReturn(['realKey' => [[], []]]);
 
         $resultCacheMock = $this->createMock(CacheItemPoolInterface::class);
 
@@ -599,7 +682,7 @@ class ConnectionTest extends TestCase
             ->with($query, $params, $types, $expectedConnectionParams)
             ->willReturn(['cacheKey', 'realKey']);
 
-        $driver = $this->createMock(Driver::class);
+        $driver = self::createStub(Driver::class);
 
         (new Connection(self::CONNECTION_PARAMS, $driver))
             ->executeCacheQuery($query, $params, $types, $queryCacheProfileMock);
@@ -623,4 +706,47 @@ class ConnectionTest extends TestCase
         $connection = DriverManager::getConnection(['driver' => 'sqlite3', 'memory' => true]);
         self::assertInstanceOf(SQLiteSchemaManager::class, $connection->createSchemaManager());
     }
+
+    public function testItPreservesTheOriginalExceptionOnRollbackFailure(): void
+    {
+        $connection = new class (['memory' => true], new Driver\SQLite3\Driver()) extends Connection {
+            public function rollBack(): void
+            {
+                throw new ConnectionException('Rollback exception');
+            }
+        };
+
+        try {
+            $connection->transactional(static function (): void {
+                throw new ConnectionException('Original exception');
+            });
+            self::fail('Exception expected'); // @phpstan-ignore deadCode.unreachable
+        } catch (ConnectionException $e) {
+            self::assertSame('Rollback exception', $e->getMessage());
+            self::assertNotNull($e->getPrevious());
+            self::assertSame('Original exception', $e->getPrevious()->getMessage());
+        }
+    }
+
+    /**
+     * We are not sure if this can happen in real life scenario
+     */
+    public function testItFailsDuringCommitBeforeTouchingDb(): void
+    {
+        $connection = new class (['memory' => true], new Driver\SQLite3\Driver()) extends Connection {
+            public function commit(): void
+            {
+                throw new \Exception('Fail before touching the db');
+            }
+
+            public function rollBack(): void
+            {
+                throw new \Exception('Rollback got triggered');
+            }
+        };
+
+        $this->expectExceptionMessage('Rollback got triggered');
+        $connection->transactional(static function (): void {
+        });
+    }
 }
