@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Doctrine\DBAL\Tests\Functional;
 
 use Doctrine\DBAL\Tests\FunctionalTestCase;
@@ -23,7 +25,7 @@ final class LikeWildcardsEscapingTest extends FunctionalTestCase
                     $escapeChar,
                 ),
             ),
-        )->execute();
+        )->executeQuery();
 
         self::assertTrue((bool) $result->fetchOne());
     }
