@@ -18,9 +18,9 @@ class MiddlewareTest extends TestCase
 
     public function setUp(): void
     {
-        $connection = $this->createMock(Connection::class);
+        $connection = self::createStub(Connection::class);
 
-        $driver = $this->createMock(Driver::class);
+        $driver = self::createStub(Driver::class);
         $driver->method('connect')
             ->willReturn($connection);
 
@@ -33,18 +33,16 @@ class MiddlewareTest extends TestCase
     public function testConnectAndDisconnect(): void
     {
         $this->driver->connect([
-            'username' => 'admin',
+            'user' => 'admin',
             'password' => 'Passw0rd!',
-            'url' => 'mysql://user:secret@localhost/mydb',
         ]);
 
         self::assertTrue($this->logger->hasInfo([
             'message' => 'Connecting with parameters {params}',
             'context' => [
                 'params' => [
-                    'username' => 'admin',
+                    'user' => 'admin',
                     'password' => '<redacted>',
-                    'url' => '<redacted>',
                 ],
             ],
         ]));
@@ -84,38 +82,20 @@ class MiddlewareTest extends TestCase
         self::assertTrue($this->logger->hasDebug('Rolling back transaction'));
     }
 
-    public function testExecuteStatementWithUntypedParameters(): void
-    {
-        $connection = $this->driver->connect([]);
-        $statement  = $connection->prepare('SELECT ?');
-        $statement->execute([42]);
-
-        self::assertTrue($this->logger->hasDebug([
-            'message' => 'Executing statement: {sql} (parameters: {params}, types: {types})',
-            'context' => [
-                'sql' => 'SELECT ?',
-                'params' => [42],
-                'types' => [],
-            ],
-        ]));
-    }
-
-    public function testExecuteStatementWithTypedParameters(): void
+    public function testExecuteStatementWithParameters(): void
     {
         $connection = $this->driver->connect([]);
         $statement  = $connection->prepare('SELECT ?, ?');
         $statement->bindValue(1, 42, ParameterType::INTEGER);
-        $statement->bindParam(2, $byRef, ParameterType::STRING);
 
-        $byRef = 'Test';
         $statement->execute();
 
         self::assertTrue($this->logger->hasDebug([
             'message' => 'Executing statement: {sql} (parameters: {params}, types: {types})',
             'context' => [
                 'sql' => 'SELECT ?, ?',
-                'params' => [1 => 42, 2 => 'Test'],
-                'types' => [1 => ParameterType::INTEGER, 2 => ParameterType::STRING],
+                'params' => [1 => 42],
+                'types' => [1 => ParameterType::INTEGER],
             ],
         ]));
     }
@@ -124,7 +104,7 @@ class MiddlewareTest extends TestCase
     {
         $connection = $this->driver->connect([]);
         $statement  = $connection->prepare('SELECT :value');
-        $statement->bindValue('value', 'Test');
+        $statement->bindValue('value', 'Test', ParameterType::STRING);
 
         $statement->execute();
 
