@@ -14,6 +14,7 @@ use Doctrine\DBAL\Schema\Column;
 use Doctrine\DBAL\Schema\ColumnDiff;
 use Doctrine\DBAL\Schema\Comparator;
 use Doctrine\DBAL\Schema\ComparatorConfig;
+use Doctrine\DBAL\Schema\DefaultExpression\CurrentDate;
 use Doctrine\DBAL\Schema\Index;
 use Doctrine\DBAL\Schema\PrimaryKeyConstraint;
 use Doctrine\DBAL\Schema\Sequence;
@@ -1087,7 +1088,7 @@ class SQLServerPlatformTest extends AbstractPlatformTestCase
                 ' DEFAULT CONVERT(date, GETDATE())',
                 $this->platform->getDefaultValueDeclarationSQL([
                     'type' => Type::getType($type),
-                    'default' => $currentDateSql,
+                    'default' => new CurrentDate(),
                 ]),
             );
         }
@@ -1140,7 +1141,11 @@ class SQLServerPlatformTest extends AbstractPlatformTestCase
 
     public function testGeneratesSequenceSqlCommands(): void
     {
-        $sequence = new Sequence('myseq', 20, 1);
+        $sequence = Sequence::editor()
+            ->setUnquotedName('myseq')
+            ->setAllocationSize(20)
+            ->create();
+
         self::assertEquals(
             'CREATE SEQUENCE myseq START WITH 1 INCREMENT BY 20 MINVALUE 1',
             $this->platform->getCreateSequenceSQL($sequence),
@@ -1225,4 +1230,13 @@ class SQLServerPlatformTest extends AbstractPlatformTestCase
             'multiple values' => [['foo', 'bar1'], 'NVARCHAR(4)'],
         ];
     }
+
+    /** @return array<string, array{array<string>, int, string}> */
+    public static function getEnumDeclarationWithLengthSQLProvider(): array
+    {
+        return [
+            'single value and bigger length' => [['foo'], 42, 'NVARCHAR(42)'],
+            'multiple values and bigger length' => [['foo', 'bar1'], 42, 'NVARCHAR(42)'],
+        ];
+    }
 }
