@@ -12,7 +12,6 @@ use Doctrine\DBAL\Platforms\MySQLPlatform;
 use Doctrine\DBAL\Schema\Column;
 use Doctrine\DBAL\Schema\MySQLSchemaManager;
 use Doctrine\DBAL\Schema\Table;
-use Doctrine\DBAL\Types\Type;
 use Doctrine\DBAL\Types\Types;
 use PHPUnit\Framework\TestCase;
 
@@ -41,15 +40,33 @@ class MySQLInheritCharsetTest extends TestCase
         $platform = new MySQLPlatform();
 
         // no options
-        $table = new Table('foobar', [new Column('aa', Type::getType(Types::INTEGER))]);
+        $table = Table::editor()
+            ->setUnquotedName('foobar')
+            ->setColumns(
+                Column::editor()
+                ->setUnquotedName('aa')
+                ->setTypeName(Types::INTEGER)
+                ->create(),
+            )
+            ->create();
+
         self::assertSame(
             ['CREATE TABLE foobar (aa INT NOT NULL)'],
             $platform->getCreateTableSQL($table),
         );
 
         // charset
-        $table = new Table('foobar', [new Column('aa', Type::getType(Types::INTEGER))]);
-        $table->addOption('charset', 'utf8');
+        $table = Table::editor()
+            ->setUnquotedName('foobar')
+            ->setColumns(
+                Column::editor()
+                    ->setUnquotedName('aa')
+                    ->setTypeName(Types::INTEGER)
+                    ->create(),
+            )
+            ->setOptions(['charset' => 'utf8'])
+            ->create();
+
         self::assertSame(
             ['CREATE TABLE foobar (aa INT NOT NULL) DEFAULT CHARACTER SET utf8'],
             $platform->getCreateTableSQL($table),
@@ -64,7 +81,7 @@ class MySQLInheritCharsetTest extends TestCase
      */
     private function getTableOptionsForOverride(array $params = []): array
     {
-        $driverMock = $this->createMock(Driver::class);
+        $driverMock = self::createStub(Driver::class);
 
         $platform = new MySQLPlatform();
         $conn     = new Connection($params, $driverMock, new Configuration());
