@@ -9,7 +9,7 @@ use Doctrine\DBAL\Platforms\AbstractPlatform;
 use Doctrine\DBAL\Types\ConversionException;
 use Doctrine\DBAL\Types\Type;
 use PHPUnit\Framework\Attributes\DataProvider;
-use PHPUnit\Framework\MockObject\MockObject;
+use PHPUnit\Framework\MockObject\Stub;
 use PHPUnit\Framework\TestCase;
 use stdClass;
 
@@ -18,7 +18,7 @@ use function date_default_timezone_set;
 
 abstract class BaseDateTypeTestCase extends TestCase
 {
-    protected AbstractPlatform&MockObject $platform;
+    protected AbstractPlatform&Stub $platform;
     protected Type $type;
 
     /** @var non-empty-string */
@@ -26,7 +26,7 @@ abstract class BaseDateTypeTestCase extends TestCase
 
     protected function setUp(): void
     {
-        $this->platform        = $this->createMock(AbstractPlatform::class);
+        $this->platform        = self::createStub(AbstractPlatform::class);
         $this->currentTimezone = date_default_timezone_get();
     }
 
